#/bin/bash
# zip-wrap.sh - bash zip,unzip,zipinfo wrapper for xarchive frontend
# Copyright (C) 2005 Lee Bigelow <ligelowbee@yahoo.com> 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# set up exit status variables
E_UNSUPPORTED=65

# Supported file extentions for zip 
EXTS="zip cbz"

# Programs to wrap
ZIP_PROG="zip"
UNZIP_PROG="unzip"
ZIPINFO_PROG="zipinfo"

# setup variables opt and archive.
# the shifting will leave the files passed as
# all the remaining args: "$@"
opt="$1"
shift 1
archive="$1"
shift 1

# Command line options for prog functions

# for zip, use recursive when adding files
NEW_OPTS="-r"
ADD_OPTS="-g -r"
REMOVE_OPTS="-d"

# for zipinfo, short list, no header, no tail
OPEN_OPTS="-s-h-t"

# for unzip, don't overwrite existing files, just skip the confilct.
EXTRACT_OPTS="-n"

# the option switches
case "$opt" in
    -i) # info: output supported extentions for progs that exist
        if [ "$(which $UNZIP_PROG)" -a "$(which $ZIPINFO_PROG)" ]; then
            for ext in $EXTS; do
                printf "%s;" $ext
            done
            if [ ! "$(which $ZIP_PROG)" ]; then
                echo warning: $ZIP_PROG not found, extract only >/dev/stderr
            fi
        else
            echo commands $UNZIP_PROG  and $ZIPINFO_PROG not found >/dev/stderr 
            echo extentions $EXTS ignored >/dev/stderr
        fi
        printf "\n"
        exit
        ;;

    -o) # open: mangle output of zipinfo cmd for xarchive 
        # format of zipinfo output:
        # -rw-r--r--  2.3 unx    11512 tx defN 28-Feb-05 16:19 file.txt 
        
        ATTR=0; SIZE=3; DATE=6; TIME=7; NAME=8; USR=GRP=LINK=9 
        # set a[9] blank for unsupported entries
        a[9]="-"

        $ZIPINFO_PROG $OPEN_OPTS "$archive" | while read dataline; do
            i=0
            for element in $dataline; do
                if [ $i -lt $NAME ]; then
                    a[i]=$element 
                    let i++
                fi
            done
            # to get name use whole data line and strip everthing before and
            # including date and a space (preserves spaces in names).
            a[NAME]="${dataline##*??:?? }" 
            # output for xarchive
            echo "${a[NAME]};${a[SIZE]};${a[ATTR]};${a[USR]};${a[GRP]};${a[DATE]};${a[TIME]};${a[LINK]}" 
        done
        exit
        ;;

    -a) # add:  to archive passed files
        # we only want to add the file's basename, not
        # the full path so...
        while [ "$1" ]; do
            cd "$(dirname "$1")"
            $ZIP_PROG $ADD_OPTS "$archive" "$(basename "$1")"
            wrapper_status=$?
            shift 1
        done
        exit $wrapper_status
        ;;

    -n) # new: create new archive with passed files 
        # create will only be passed the first file, the
        # rest will be "added" to the new archive
        cd "$(dirname "$1")"
        $ZIP_PROG $NEW_OPTS "$archive" "$(basename "$1")"
        exit
        ;;

    -r) # remove: from archive passed files 
        $ZIP_PROG $REMOVE_OPTS "$archive" "$@"
        exit
        ;;

    -e) # extract: from archive passed files 
        # xarchive will put is the right extract dir
        # so we just have to extract.
        $UNZIP_PROG $EXTRACT_OPTS "$archive" "$@"
        exit
        ;;

    -v) # view: from archive passed files 
        exit $E_UNSUPPORTED
        ;;

     *) echo "error, option $opt not supported"
        echo "use one of these:" 
        echo "-i                #info" 
        echo "-o archive        #open" 
        echo "-a archive files  #add" 
        echo "-n archive file   #new" 
        echo "-r archive files  #remove" 
        echo "-e archive files  #extract" 
        echo "-v archive file   #view"  
        exit
esac
